#!/usr/bin/perl
use Crypt::Blowfish;
use Digest::MD5 qw/ md5_hex /;

my $path = "/usr/local/apache/htdocs/audio";
my $pathurl = "/audio";

$vmroot = "/var/spool/asterisk/voicemail/default";

$string = $ENV{QUERY_STRING};
$string =~ s/\.wav//g;
my ($vm_format) = $ENV{QUERY_STRING} =~ m/vm_format=(\w+)/;
# remove vm_format query parts so $string is as originally intended.
$string =~ s/\&vm_format=\w+//g;

open(KEY, "</etc/asterisk/key") || die "can't open key file: $!";
chomp(my $key = <KEY>);
close(KEY);

$key = pack("H78", $key);
my $cipher = new Crypt::Blowfish $key;

my @blocks = unpack("a16" x (int(length($string) / 8)+1),$string);
my $text;
foreach my $block (@blocks)
{
   last unless $block;
   $text .= $cipher->decrypt(pack("H16", $block));
	$text =~ s/[^\w\s\-\,\.\/\:\?\`]//g;
}
$text =~ s/\s*$//;

if ($text =~ /^Delete`/){   # delete the message
   ($action, $redirect, $file_loc)  = split(/`/, $text);
   $redirect = $ENV{HTTP_REFERER};
	$file = "$vmroot/$file_loc";
	
	if (-e $file && $text ne ""){
	    $filetxt = $file;
		 $filetxt =~ s/\.wav/\.txt/;
		 if (unlink $file){  #remove the wav file
	        if (unlink $filetxt){ # remove the txt file
		 	 		print "Location: $redirect\n\n";

		 		} else {
		 	 		print "Content-type: text/html\n\n";
		    		print "Can not remove $filetxt\n";
		 		}
		 } else {
		 	print "Content-type: text/html\n\n";
			print "Can not remove $file\n";
		}

	} else {
		print "Content-type: text/html\n";
		print "Can not find file $file\n";
	}
}
# move the message
elsif ($text =~ /^(?:Save|Move)`/)
{
	my ($action, $redirect, $file_loc, $to) = split(/`/, $text);
        $redirect = $ENV{HTTP_REFERER};
	my $file = "$vmroot/$file_loc";

	if (-e $file)
	{
		my $filetxt = $file;
		$filetxt =~ s/\.wav/\.txt/;

		my $newfile = $file;
		my $newfiletxt = $filetxt;

		$newfile    =~ s| ^ (.*/) [^/]+ (/[^/]+\.wav) $ |$1$to$2|x;
		$newfiletxt =~ s| ^ (.*/) [^/]+ (/[^/]+\.txt) $ |$1$to$2|x;

		unless (-e $1 . $to)
		{
			mkdir($1 . $to, 0755);
		}
		rename($file, $newfile);
		rename($filetxt, $newfiletxt);

 		print "Location: $redirect\n\n";

	}
	else
	{
		print "Content-type: text/html\n";
		print "Can not find file $file\n";
	}
}
# play the message
else
{
	my $file = "$vmroot/$text";

	if (-e $file && $text ne "")
	{
		if ($vm_format eq 'mp3') {
			if (!md5path_mp3($file)) {
				print "Content-Type: text/html\n\n";
				print "<b>Error Message:</b> Could not playback file: $file<br>\n<b>Error Type:</b> MD5 Path Creation\n";
			}
			else {
				print "Location: " . md5path_mp3($file) . "\r\n\r\n";
			}
	    }
	    else {
		  print "Location: " . randpath($file) . "\r\n\r\n";
	    }
	}
	else
	{
		print "Content-Type: text/html\n\n";
		print "File not found ^$file^\n";
	}
}


sub randpath
{
	my $wav = shift;
	my $rand = random_characters();
	link($wav, "$path/$rand.wav");
	chmod(0755, "$path/$rand.wav");

	return "$pathurl/$rand.wav";
}

# Create an .mp3 from a .wav and return its location.
sub randpath_mp3
{
        my $wav = shift;

        my $rand = random_characters();
        link($wav, "$path/$rand.wav");
        chmod(0755, "$path/$rand.wav");
        # Resample encoding to 16 kHz for better voice quality.
        system("lame --quiet --resample 16 $path/$rand.wav");

        return "$pathurl/$rand.wav.mp3";
}

=pod

Create an .mp3 from a .wav and return its location.

	Args: path of wav voicemail file
	Returns: URL to an mp3 version of the voicemail or undef

=cut

sub md5path_mp3
{
	my $wav = shift;

	my $md5_hex;
	return if !($md5_hex = md5_hex_head($wav));
	my $wav_tmp = "$path/${md5_hex}.wav";
	if (! -e  $wav_tmp) {
		link($wav, $wav_tmp);
		chmod(0755, $wav_tmp);
	}
	if (! -e  "${wav_tmp}.mp3") {
		# Resample encoding to 16 kHz for better voice quality.
		system("lame --quiet --resample 16 ${wav_tmp}");
	}
    return "$pathurl/${md5_hex}.wav.mp3";
}
# Create a random string of characters with default length of 12.
sub random_characters {
    my $number_of_characters = shift || 12;

    my @chars = ("a".."z","A".."Z",0..9);
	my $rand;
	foreach (1 .. $number_of_characters)
	{
		$rand .= $chars[int(rand(@chars))];
	}

	return $rand;
}

=pod

 Compute md5 on the first 300kb of file
 or on the whole file when the file size is < 300kb.
 
 Args: $file_name
 Returns: md5_hex of first 300kb of file on success otherwise false 

=cut

sub md5_hex_head
{
    my $file = shift;

    my $length = 307200;
    my $buffer = '';
    open(F, '<', $file) || return;
    my $bytes_read = sysread(F, $buffer, $length);
    close F;

    return md5_hex($buffer);
}


